struct Builder<R: Runtime, C: Buildable + Sync + Send + 'static> {
runtime: R,
chanmgr: Arc<ChanMgr<R>>,
timeouts: Arc<Estimator>,
_phantom: PhantomData<C>,
}Expand description
An implementation type for TunnelBuilder.
A TunnelBuilder holds references to all the objects that are needed
to build circuits correctly.
In general, you should not need to construct or use this object yourself, unless you are choosing your own paths.
Fields
runtime: RThe runtime used by this circuit builder.
chanmgr: Arc<ChanMgr<R>>A channel manager that this circuit builder uses to make channels.
timeouts: Arc<Estimator>An estimator to determine the correct timeouts for circuit building.
_phantom: PhantomData<C>We don’t actually hold any clientcircs, so we need to put this type here so the compiler won’t freak out.
Implementations
Sourceimpl<R: Runtime, C: Buildable + Sync + Send + 'static> Builder<R, C>
impl<R: Runtime, C: Buildable + Sync + Send + 'static> Builder<R, C>
Sourcefn new(runtime: R, chanmgr: Arc<ChanMgr<R>>, timeouts: Estimator) -> Self
fn new(runtime: R, chanmgr: Arc<ChanMgr<R>>, timeouts: Estimator) -> Self
Construct a new Builder.
Sourceasync fn build_notimeout(
self: Arc<Self>,
path: OwnedPath,
channel: Arc<C::Chan>,
params: CircParameters,
start_time: Instant,
n_hops_built: Arc<AtomicU32>,
guard_status: Arc<GuardStatusHandle>,
) -> Result<C>
async fn build_notimeout( self: Arc<Self>, path: OwnedPath, channel: Arc<C::Chan>, params: CircParameters, start_time: Instant, n_hops_built: Arc<AtomicU32>, guard_status: Arc<GuardStatusHandle>, ) -> Result<C>
Build a circuit, without performing any timeout operations.
After each hop is built, increments n_hops_built. Make sure that
guard_status has its pending status set correctly to correspond
to a circuit failure at any given stage.
Requires that channel is a channel to the first hop of path.
(TODO: Find a better design there.)
Sourceasync fn build_owned(
self: &Arc<Self>,
path: OwnedPath,
params: &CircParameters,
guard_status: Arc<GuardStatusHandle>,
usage: ChannelUsage,
) -> Result<C>
async fn build_owned( self: &Arc<Self>, path: OwnedPath, params: &CircParameters, guard_status: Arc<GuardStatusHandle>, usage: ChannelUsage, ) -> Result<C>
Build a circuit from an OwnedPath.