CircuitTimingBuilder

Struct CircuitTimingBuilder 

pub struct CircuitTimingBuilder {
    max_dirtiness: Option<Duration>,
    disused_circuit_timeout: Option<Duration>,
    request_timeout: Option<Duration>,
    request_max_retries: Option<u32>,
    request_loyalty: Option<Duration>,
    hs_desc_fetch_attempts: Option<u32>,
    hs_intro_rend_attempts: Option<u32>,
}
A builder to create an instance of CircuitTiming.

Fields

max_dirtiness: Option<Duration>

In-progress value for max_dirtiness.

See CircuitTiming.max_dirtiness

disused_circuit_timeout: Option<Duration>

In-progress value for disused_circuit_timeout.

See CircuitTiming.disused_circuit_timeout

request_timeout: Option<Duration>

In-progress value for request_timeout.

See CircuitTiming.request_timeout

request_max_retries: Option<u32>

In-progress value for request_max_retries.

See CircuitTiming.request_max_retries

request_loyalty: Option<Duration>

In-progress value for request_loyalty.

See CircuitTiming.request_loyalty

hs_desc_fetch_attempts: Option<u32>

In-progress value for hs_desc_fetch_attempts.

See CircuitTiming.hs_desc_fetch_attempts

hs_intro_rend_attempts: Option<u32>

In-progress value for hs_intro_rend_attempts.

See CircuitTiming.hs_intro_rend_attempts

Implementations

Source

impl CircuitTimingBuilder

pub fn max_dirtiness(&mut self, val: Duration) -> &mut Self

Provide a value for max_dirtiness.

If no value is provided for max_dirtiness, build will use Default::default().

max_dirtiness

How long after a circuit has first been used should we give it out for new requests?

This setting applies to circuits without strong isolation. See also disused_circuit_timeout

pub fn disused_circuit_timeout(&mut self, val: Duration) -> &mut Self

Provide a value for disused_circuit_timeout.

If no value is provided for disused_circuit_timeout, build will use Default::default().

disused_circuit_timeout

How long after a circuit has become disused should we discard it?

This setting applies to circuits with strong isolation. See also max_dirtiness

pub fn request_timeout(&mut self, val: Duration) -> &mut Self

Provide a value for request_timeout.

If no value is provided for request_timeout, build will use Default::default().

request_timeout

When a circuit is requested, we stop retrying new circuits after this much time.

pub fn request_max_retries(&mut self, val: u32) -> &mut Self

Provide a value for request_max_retries.

If no value is provided for request_max_retries, build will use Default::default().

request_max_retries

When a circuit is requested, we stop retrying new circuits after this many attempts.

pub fn request_loyalty(&mut self, val: Duration) -> &mut Self

Provide a value for request_loyalty.

If no value is provided for request_loyalty, build will use Default::default().

request_loyalty

When waiting for requested circuits, wait at least this long before using a suitable-looking circuit launched by some other request.

pub fn hs_desc_fetch_attempts(&mut self, val: u32) -> &mut Self

Provide a value for hs_desc_fetch_attempts.

If no value is provided for hs_desc_fetch_attempts, build will use Default::default().

hs_desc_fetch_attempts

When an HS connection is attempted, we stop trying more hsdirs after this many attempts

pub fn hs_intro_rend_attempts(&mut self, val: u32) -> &mut Self

Provide a value for hs_intro_rend_attempts.

If no value is provided for hs_intro_rend_attempts, build will use Default::default().

hs_intro_rend_attempts

When an HS connection is attempted, we stop trying intro/rendezvous after this many attempts

Source

impl CircuitTimingBuilder

pub fn new() -> Self

Return a new builder object.

pub fn build(&self) -> Result<CircuitTiming, ConfigBuildError>

Try to construct a new CircuitTiming from the fields set in this builder.

Return an error if any required field is missing, or is set to something invalid.