pub struct PreemptiveCircuitConfig {
pub(crate) disable_at_threshold: usize,
pub(crate) initial_predicted_ports: Vec<u16>,
pub(crate) prediction_lifetime: Duration,
pub(crate) min_exit_circs_for_port: usize,
}Expand description
Configuration for preemptive circuits.
Preemptive circuits are built ahead of time, to anticipate client need. This object configures the way in which this demand is anticipated and in which these circuits are constructed.
This type is immutable once constructed. To create an object of this type,
use PreemptiveCircuitConfigBuilder.
Except as noted, this configuration can be changed on a running Arti client.
Fields
disable_at_threshold: usizeIf we have at least this many available circuits, we suspend construction of preemptive circuits. whether our available circuits support our predicted exit ports or not.
initial_predicted_ports: Vec<u16>At startup, which exit ports should we expect that the client will want?
(Over time, new ports are added to the predicted list, in response to what the client has actually requested.)
This value cannot be changed on a running Arti client, because doing so would be meaningless.
The default is [80, 443].
prediction_lifetime: DurationAfter we see the client request a connection to a new port, how long should we predict that the client will still want to have circuits available for that port?
min_exit_circs_for_port: usizeHow many available circuits should we try to have, at minimum, for each predicted exit port?
Implementations
Sourceimpl PreemptiveCircuitConfig
impl PreemptiveCircuitConfig
Sourcepub fn builder() -> PreemptiveCircuitConfigBuilder
pub fn builder() -> PreemptiveCircuitConfigBuilder
Return a new [PreemptiveCircuitConfig Builder] to construct an instance of this type.