Enum tor_circmgr::DirInfo

source ·
#[non_exhaustive]
pub enum DirInfo<'a> {
    Fallbacks(&'a FallbackList),
    Directory(&'a NetDir),
    Nothing,
}
Represents what we know about the Tor network.

This can either be a complete directory, or a list of fallbacks.

Not every DirInfo can be used to build every kind of circuit: if you try to build a path with an inadequate DirInfo, you’ll get a NeedConsensus error.

Variants (Non-exhaustive)

This enum is marked as non-exhaustive
Non-exhaustive enums could have additional variants added in future. Therefore, when matching against variants of non-exhaustive enums, an extra wildcard arm must be added to account for any future variants.


Fallbacks(&'a FallbackList)

A list of fallbacks, for use when we don’t know a network directory.



Directory(&'a NetDir)

A complete network directory



Nothing

No information: we can only build one-hop paths: and that, only if the guard manager knows some guards or fallbacks.