Enum tor_circmgr::ExternalActivity
source · #[non_exhaustive]
pub enum ExternalActivity {
DirCache,
}Expand description
An activity that can succeed or fail, and whose success or failure can be attributed to a guard.
Variants (Non-exhaustive)
This enum is marked as non-exhaustive
Non-exhaustive enums could have additional variants added in future. Therefore, when matching against variants of non-exhaustive enums, an extra wildcard arm must be added to account for any future variants.
DirCache
The activity of using the guard as a directory cache.