circuit_compatible_with_target

Function circuit_compatible_with_target 

fn circuit_compatible_with_target<C: AbstractTunnel>(
    netdir: &NetDir,
    circ: &HsCircStem<C>,
    circ_kind: HsCircKind,
    exclude_target: &RelayExclusion<'_>,
) -> bool
Return true if we can extend a pre-built circuit circ to target.

We require that the circuit is open, that every hop in the circuit is listed in netdir, and that no hop in the circuit shares a family with target.