circuit_still_useable

Function circuit_still_useable 

fn circuit_still_useable<C, F1, F2>(
    netdir: &NetDir,
    circ: &HsCircStem<C>,
    relay_okay: F1,
    last_hop_ok: F2,
) -> bool
where C: AbstractTunnel, F1: Fn(&Relay<'_>) -> bool, F2: Fn(&Relay<'_>) -> bool,
Return true if we can still use a given pre-build circuit.

We require that the circuit is open, that every hop in the circuit is listed in netdir, and that relay_okay returns true for every hop on the circuit.