vanguards_circuit_compatible_with_target

Function vanguards_circuit_compatible_with_target 

fn vanguards_circuit_compatible_with_target<C: AbstractTunnel, T>(
    netdir: &NetDir,
    circ: &HsCircStem<C>,
    kind: HsCircStemKind,
    circ_kind: HsCircKind,
    avoid_target: Option<&T>,
) -> bool
where T: CircTarget + Sync,
Return true if we can extend a pre-built vanguards circuit circ to target.

We require that the circuit is open, that it can become the specified kind of HsCircStem, that every hop in the circuit is listed in netdir, and that the last two hops are different from the specified target.