pub(crate) struct HsCircPoolInner<B: AbstractTunnelBuilder<R> + 'static, R: Runtime> {
circmgr: Arc<CircMgrInner<B, R>>,
launcher_handle: OnceCell<TaskHandle>,
inner: Mutex<Inner<B::Tunnel>>,
}Expand description
An object to provide circuits for implementing onion services.
Fields
circmgr: Arc<CircMgrInner<B, R>>An underlying circuit manager, used for constructing circuits.
launcher_handle: OnceCell<TaskHandle>A task handle for making the background circuit launcher fire early.
inner: Mutex<Inner<B::Tunnel>>The mutable state of this pool.
Implementations
Sourceimpl<R: Runtime> HsCircPoolInner<TunnelBuilder<R>, R>
impl<R: Runtime> HsCircPoolInner<TunnelBuilder<R>, R>
Sourcepub(crate) fn new(circmgr: &CircMgr<R>) -> Self
pub(crate) fn new(circmgr: &CircMgr<R>) -> Self
Internal implementation for HsCircPool::new.
Sourceimpl<B: AbstractTunnelBuilder<R> + 'static, R: Runtime> HsCircPoolInner<B, R>
impl<B: AbstractTunnelBuilder<R> + 'static, R: Runtime> HsCircPoolInner<B, R>
Sourcepub(crate) fn new_internal(circmgr: &Arc<CircMgrInner<B, R>>) -> Self
pub(crate) fn new_internal(circmgr: &Arc<CircMgrInner<B, R>>) -> Self
Create a new HsCircPoolInner from a CircMgrInner.
Sourcepub(crate) fn launch_background_tasks(
self: &Arc<Self>,
runtime: &R,
netdir_provider: &Arc<dyn NetDirProvider + 'static>,
) -> Result<Vec<TaskHandle>>
pub(crate) fn launch_background_tasks( self: &Arc<Self>, runtime: &R, netdir_provider: &Arc<dyn NetDirProvider + 'static>, ) -> Result<Vec<TaskHandle>>
Internal implementation for HsCircPool::launch_background_tasks.
Sourcepub(crate) async fn get_or_launch_client_rend<'a>(
&self,
netdir: &'a NetDir,
) -> Result<(B::Tunnel, Relay<'a>)>
pub(crate) async fn get_or_launch_client_rend<'a>( &self, netdir: &'a NetDir, ) -> Result<(B::Tunnel, Relay<'a>)>
Internal implementation for HsCircPool::get_or_launch_client_rend.
Sourcepub(crate) async fn get_or_launch_specific<T>(
&self,
netdir: &NetDir,
kind: HsCircKind,
target: T,
) -> Result<B::Tunnel>where
T: CircTarget + Sync,
pub(crate) async fn get_or_launch_specific<T>(
&self,
netdir: &NetDir,
kind: HsCircKind,
target: T,
) -> Result<B::Tunnel>where
T: CircTarget + Sync,
Helper for the HsCircPool functions that launch rendezvous,
introduction, or directory circuits.
Sourceasync fn extend_circ<T>(
&self,
circ: HsCircStem<B::Tunnel>,
params: CircParameters,
target: T,
) -> Result<B::Tunnel>where
T: CircTarget + Sync,
async fn extend_circ<T>(
&self,
circ: HsCircStem<B::Tunnel>,
params: CircParameters,
target: T,
) -> Result<B::Tunnel>where
T: CircTarget + Sync,
Try to extend a circuit to the specified target hop.
Sourcepub(crate) fn retire_all_circuits(&self) -> StdResult<(), ReconfigureError>
pub(crate) fn retire_all_circuits(&self) -> StdResult<(), ReconfigureError>
Internal implementation for HsCircPool::retire_all_circuits.
Sourceasync fn take_or_launch_stem_circuit<T>(
&self,
netdir: &NetDir,
avoid_target: Option<&T>,
kind: HsCircKind,
) -> Result<HsCircStem<B::Tunnel>>where
T: CircTarget + Sync,
async fn take_or_launch_stem_circuit<T>(
&self,
netdir: &NetDir,
avoid_target: Option<&T>,
kind: HsCircKind,
) -> Result<HsCircStem<B::Tunnel>>where
T: CircTarget + Sync,
Take and return a circuit from our pool suitable for being extended to avoid_target.
If vanguards are enabled, this will try to build a circuit stem appropriate for use
as the specified kind.
If vanguards are disabled, kind is unused.
If there is no such circuit, build and return a new one.
Sourceasync fn maybe_extend_stem_circuit<T>(
&self,
netdir: &NetDir,
circuit: HsCircStem<B::Tunnel>,
avoid_target: Option<&T>,
stem_kind: HsCircStemKind,
circ_kind: HsCircKind,
) -> Result<HsCircStem<B::Tunnel>>where
T: CircTarget + Sync,
async fn maybe_extend_stem_circuit<T>(
&self,
netdir: &NetDir,
circuit: HsCircStem<B::Tunnel>,
avoid_target: Option<&T>,
stem_kind: HsCircStemKind,
circ_kind: HsCircKind,
) -> Result<HsCircStem<B::Tunnel>>where
T: CircTarget + Sync,
Return a circuit of the specified kind, built from circuit.
Sourceasync fn extend_full_vanguards_circuit<T>(
&self,
netdir: &NetDir,
circuit: HsCircStem<B::Tunnel>,
avoid_target: Option<&T>,
stem_kind: HsCircStemKind,
circ_kind: HsCircKind,
) -> Result<HsCircStem<B::Tunnel>>where
T: CircTarget + Sync,
async fn extend_full_vanguards_circuit<T>(
&self,
netdir: &NetDir,
circuit: HsCircStem<B::Tunnel>,
avoid_target: Option<&T>,
stem_kind: HsCircStemKind,
circ_kind: HsCircKind,
) -> Result<HsCircStem<B::Tunnel>>where
T: CircTarget + Sync,
Extend the specified full vanguard circuit if necessary.
Sourcefn ensure_suitable_circuit<T>(
&self,
circ: &B::Tunnel,
target: Option<&T>,
kind: HsCircStemKind,
) -> Result<()>where
T: CircTarget + Sync,
fn ensure_suitable_circuit<T>(
&self,
circ: &B::Tunnel,
target: Option<&T>,
kind: HsCircStemKind,
) -> Result<()>where
T: CircTarget + Sync,
Ensure circ is compatible with target, and has the correct length for its kind.
Sourcefn ensure_circuit_length_valid(
&self,
tunnel: &B::Tunnel,
kind: HsCircStemKind,
) -> Result<()>
fn ensure_circuit_length_valid( &self, tunnel: &B::Tunnel, kind: HsCircStemKind, ) -> Result<()>
Ensure the specified circuit of type kind has the right length.
Sourcefn ensure_circuit_can_extend_to_target<T>(
tunnel: &B::Tunnel,
target: Option<&T>,
) -> Result<()>where
T: CircTarget + Sync,
fn ensure_circuit_can_extend_to_target<T>(
tunnel: &B::Tunnel,
target: Option<&T>,
) -> Result<()>where
T: CircTarget + Sync,
Ensure that it is possible to extend circ to target.
Returns an error if either of the last 2 hops of the circuit are the same as target,
because:
- a relay won’t let you extend the circuit to itself
- relays won’t let you extend the circuit to their previous hop
Sourcefn remove_closed(&self)
fn remove_closed(&self)
Internal: Remove every closed circuit from this pool.
Sourcefn remove_unlisted(&self, netdir: &NetDir)
fn remove_unlisted(&self, netdir: &NetDir)
Internal: Remove every circuit form this pool for which any relay is not
listed in netdir.
Sourcefn vanguard_mode(&self) -> VanguardMode
fn vanguard_mode(&self) -> VanguardMode
Returns the current VanguardMode.
Sourcepub(crate) fn estimate_timeout(&self, timeout_action: &Action) -> Duration
pub(crate) fn estimate_timeout(&self, timeout_action: &Action) -> Duration
Internal implementation for HsCircPool::estimate_timeout.