Struct tor_circmgr::isolation::StreamIsolation
source · pub struct StreamIsolation { /* private fields */ }Expand description
A set of information about how a stream should be isolated.
If two streams are isolated from one another, they may not share a circuit.
Implementations
sourceimpl StreamIsolation
impl StreamIsolation
sourcepub fn no_isolation() -> Self
pub fn no_isolation() -> Self
Construct a new StreamIsolation with no isolation enabled.
sourcepub fn builder() -> StreamIsolationBuilder
pub fn builder() -> StreamIsolationBuilder
Return a new StreamIsolationBuilder for constructing StreamIsolation objects.