enum ExpirationInfo {
Unused {
created: Instant,
},
Dirty {
dirty_since: Instant,
},
LongLived {
last_known_to_be_used_at: Instant,
},
}Expand description
Enumeration to track the expiration state of a tunnel.
A tunnel an either be unused (at which point it should expire if it is still unused by a certain time, or dirty (at which point it should expire after a certain duration).
All tunnels start out “unused” and become “dirty” when their spec is first restricted – that is, when they are first handed out to be used for a request.
Variants
Unused
The tunnel has never been used, and has never been restricted for use with a request.
Dirty
The tunnel is not-long-lived; we will expire by waiting until a certain amount of time after it was first used.
LongLived
The tunnel is long-lived; we will expire by waiting until it has passed a certain amount of time without having any streams attached to it.
Implementations
Sourceimpl ExpirationInfo
impl ExpirationInfo
Sourcefn mark_used(&mut self, now: Instant, long_lived: bool)
fn mark_used(&mut self, now: Instant, long_lived: bool)
Mark this ExpirationInfo as having been in-use at now.
If long_lived is false, the associated tunnel should expire a certain amount of time
after it was first used.
If long_lived is true, the associated tunnel should expire a certain amount of time
after it was last used.
Sourcefn check_long_lived(&self) -> Result<()>
fn check_long_lived(&self) -> Result<()>
Return an internal error if this ExpirationInfo is not marked as long-lived.