spawn_expiration_task

Function spawn_expiration_task 

fn spawn_expiration_task<B, R>(
    runtime: &R,
    circmgr: Weak<AbstractTunnelMgr<B, R>>,
    circ_id: <<B as AbstractTunnelBuilder<R>>::Tunnel as AbstractTunnel>::Id,
    exp_inst: Instant,
)
where R: Runtime, B: 'static + AbstractTunnelBuilder<R>,
Spawn an expiration task that expires a tunnel at given instant.

When the timeout occurs, if the tunnel manager is still present, the task will ask the manager to expire the tunnel, if the tunnel is ready to expire.