pub(crate) struct OpenEntry<T> {
spec: SupportedTunnelUsage,
tunnel: Arc<T>,
expiration: ExpirationInfo,
}Expand description
An entry for an open tunnel held by an AbstractTunnelMgr.
Fields
spec: SupportedTunnelUsageThe supported usage for this tunnel.
tunnel: Arc<T>The tunnel under management.
expiration: ExpirationInfoWhen does this tunnel expire?
(Note that expired tunnels are removed from the manager, which does not actually close them until there are no more references to them.)
Implementations
Sourceimpl<T: AbstractTunnel> OpenEntry<T>
impl<T: AbstractTunnel> OpenEntry<T>
Sourcefn new(
spec: SupportedTunnelUsage,
tunnel: T,
expiration: ExpirationInfo,
) -> Self
fn new( spec: SupportedTunnelUsage, tunnel: T, expiration: ExpirationInfo, ) -> Self
Make a new OpenEntry for a given tunnel and spec.
Sourcepub(crate) fn supports(&self, usage: &TargetTunnelUsage) -> bool
pub(crate) fn supports(&self, usage: &TargetTunnelUsage) -> bool
Return true if the underlying tunnel can be used for usage.
Sourcefn restrict_mut(
&mut self,
usage: &TargetTunnelUsage,
now: Instant,
) -> Result<()>
fn restrict_mut( &mut self, usage: &TargetTunnelUsage, now: Instant, ) -> Result<()>
Change the underlying tunnel’s permissible usage, based on its having
been used for usage at time now.
Return an error if the tunnel may not be used for usage.
Sourcefn find_best<'a>(
ents: &'a mut [&'a mut Self],
usage: &TargetTunnelUsage,
parallelism: usize,
) -> &'a mut Self
fn find_best<'a>( ents: &'a mut [&'a mut Self], usage: &TargetTunnelUsage, parallelism: usize, ) -> &'a mut Self
Find the “best” entry from a slice of OpenEntry for supporting
a given usage.
If parallelism is some N greater than 1, we pick randomly
from the best N tunnels.
Requirements
Requires that ents is nonempty, and that every element of ents
supports spec.
Sourcefn should_expire(
&self,
now: Instant,
params: &ExpirationParameters,
) -> ShouldExpire
fn should_expire( &self, now: Instant, params: &ExpirationParameters, ) -> ShouldExpire
Return true if this tunnel should be expired given that the current time is now,
and the current settings are params.