struct PendingEntry<B: AbstractTunnelBuilder<R>, R: Runtime> {
tentative_assignment: Mutex<SupportedTunnelUsage>,
receiver: Shared<Receiver<Result<<<B as AbstractTunnelBuilder<R>>::Tunnel as AbstractTunnel>::Id>>>,
}Expand description
An entry for an under-construction in-progress tunnel tracked by
an AbstractTunnelMgr.
Fields
tentative_assignment: Mutex<SupportedTunnelUsage>Specification that this tunnel will support, if every pending request that is waiting for it is attached to it.
This spec becomes more and more restricted as more pending requests are waiting for this tunnel.
This spec is contained by circ_spec, and must support the usage of every pending request that’s waiting for this tunnel.
receiver: Shared<Receiver<Result<<<B as AbstractTunnelBuilder<R>>::Tunnel as AbstractTunnel>::Id>>>A shared future for requests to use when waiting for notification of this tunnel’s success.
Implementations
Sourceimpl<B: AbstractTunnelBuilder<R>, R: Runtime> PendingEntry<B, R>
impl<B: AbstractTunnelBuilder<R>, R: Runtime> PendingEntry<B, R>
Sourcefn new(
spec: &SupportedTunnelUsage,
) -> (Self, Sender<Result<<<B as AbstractTunnelBuilder<R>>::Tunnel as AbstractTunnel>::Id>>)
fn new( spec: &SupportedTunnelUsage, ) -> (Self, Sender<Result<<<B as AbstractTunnelBuilder<R>>::Tunnel as AbstractTunnel>::Id>>)
Make a new PendingEntry that starts out supporting a given spec. Return that PendingEntry, along with a Sender to use to report the result of building this tunnel.
Sourcefn supports(&self, usage: &TargetTunnelUsage) -> bool
fn supports(&self, usage: &TargetTunnelUsage) -> bool
Return true if this tunnel’s current tentative assignment
supports usage.
Sourcefn tentative_restrict_mut(&self, usage: &TargetTunnelUsage) -> Result<()>
fn tentative_restrict_mut(&self, usage: &TargetTunnelUsage) -> Result<()>
Try to change the tentative assignment of this tunnel by
restricting it for use with usage.
Return an error if the current tentative assignment didn’t
support usage in the first place.