pub(crate) trait MockablePlan {
// Provided method
fn add_blocked_advance_reason(&mut self, _reason: String) { ... }
}Expand description
A plan for an AbstractCircBuilder that can maybe be mutated by tests.
You should implement this trait using all default methods for all code that isn’t test code.
Provided Methods
Sourcefn add_blocked_advance_reason(&mut self, _reason: String)
fn add_blocked_advance_reason(&mut self, _reason: String)
Add a reason string that was passed to SleepProvider::block_advance() to this object
so that it knows what to pass to ::release_advance().
Dyn Compatibility
This trait is dyn compatible.
In older versions of Rust, dyn compatibility was called "object safety".