PtrWeakHashSet

Type Alias PtrWeakHashSet 

type PtrWeakHashSet<T> = PtrWeakHashSet<T, RandomState>;
Alias to force use of RandomState, regardless of features enabled in weak_tables.

See https://github.com/tov/weak-table-rs/issues/23 for discussion.

(We could probably get away with a weaker hash function in this case, since the attacker probably doesn’t have control over our pointers.)

Aliased Type

struct PtrWeakHashSet<T>(/* private fields */);