hs_intermediate_hop_usage

Function hs_intermediate_hop_usage 

pub(crate) fn hs_intermediate_hop_usage() -> RelayUsage
Return the usage that we should use when selecting an intermediary hop (vanguard or middle) of an HS circuit or stem circuit.

(This isn’t called “middle hop”, since we want to avoid confusion with the M hop in vanguard circuits.)