pub(crate) struct HsPathBuilder {
compatible_with: Option<OwnedChanTarget>,
stem_kind: HsCircStemKind,
circ_kind: Option<HsCircKind>,
}Expand description
A path builder for hidden service circuits.
See the hspath docs for more details.
Fields
compatible_with: Option<OwnedChanTarget>If present, a “target” that every chosen relay must be able to share a circuit with with.
Ignored if vanguards are in use.
stem_kind: HsCircStemKindThe type of circuit stem to build.
This is only used if vanguards are enabled.
circ_kind: Option<HsCircKind>If present, ensure that the circuit stem is suitable for use as (a stem for) the given kind of circuit.
Implementations
Sourceimpl HsPathBuilder
impl HsPathBuilder
Sourcepub(crate) fn new(
compatible_with: Option<OwnedChanTarget>,
stem_kind: HsCircStemKind,
circ_kind: Option<HsCircKind>,
) -> Self
pub(crate) fn new( compatible_with: Option<OwnedChanTarget>, stem_kind: HsCircStemKind, circ_kind: Option<HsCircKind>, ) -> Self
Create a new builder that will try to build a three-hop non-exit path for use with the onion services protocols that is compatible with being extended to an optional given relay.
(The provided relay is not included in the built path: we only ensure that the path we build does not have any features that would stop us extending it to that relay as a fourth hop.)
Sourcepub(crate) fn pick_path<'a, R: Rng, RT: Runtime>(
&self,
rng: &mut R,
netdir: DirInfo<'a>,
guards: &GuardMgr<RT>,
config: &PathConfig,
now: SystemTime,
) -> Result<(TorPath<'a>, GuardMonitor, GuardUsable)>
pub(crate) fn pick_path<'a, R: Rng, RT: Runtime>( &self, rng: &mut R, netdir: DirInfo<'a>, guards: &GuardMgr<RT>, config: &PathConfig, now: SystemTime, ) -> Result<(TorPath<'a>, GuardMonitor, GuardUsable)>
Try to create and return a path for a hidden service circuit stem.
Sourcepub(crate) fn pick_path_with_vanguards<'a, R: Rng, RT: Runtime>(
&self,
rng: &mut R,
netdir: DirInfo<'a>,
guards: &GuardMgr<RT>,
vanguards: &VanguardMgr<RT>,
config: &PathConfig,
now: SystemTime,
) -> Result<(TorPath<'a>, GuardMonitor, GuardUsable)>
pub(crate) fn pick_path_with_vanguards<'a, R: Rng, RT: Runtime>( &self, rng: &mut R, netdir: DirInfo<'a>, guards: &GuardMgr<RT>, vanguards: &VanguardMgr<RT>, config: &PathConfig, now: SystemTime, ) -> Result<(TorPath<'a>, GuardMonitor, GuardUsable)>
Try to create and return a path for a hidden service circuit stem.
If vanguards are disabled, this has the same behavior as pick_path.