fn exclude_neighbors<'n, T: HasRelayIds + 'n>(hops: &[T]) -> RelayExclusion<'n>Expand description
Create a RelayExclusion suitable for selecting the next hop to add to hops.
fn exclude_neighbors<'n, T: HasRelayIds + 'n>(hops: &[T]) -> RelayExclusion<'n>Create a RelayExclusion suitable for selecting the next hop to add to hops.