fn selector_excluding_neighbors<'n, T: HasRelayIds + 'n>(
selector: &RelaySelector<'n>,
hops: &[T],
) -> RelaySelector<'n>Expand description
Extend the selector T to also exclude neighbors, based on hops.
fn selector_excluding_neighbors<'n, T: HasRelayIds + 'n>(
selector: &RelaySelector<'n>,
hops: &[T],
) -> RelaySelector<'n>Extend the selector T to also exclude neighbors, based on hops.