pub(super) struct PathBuilder<'n, 'a, RT: Runtime, R: Rng> {
hops: Vec<MaybeOwnedRelay<'n>>,
netdir: &'n NetDir,
vanguards: &'a VanguardMgr<RT>,
rng: &'a mut R,
last_hop_kind: HopKind,
}Expand description
A vanguard path builder.
A PathBuilder is a state machine whose current state is the HopKind of its last hop.
Not all state transitions are valid. For the permissible state transitions, see
update_last_hop_kind.
This type is an implementation detail that should remain private.
Used by VanguardHsPathBuilder.
Fields
hops: Vec<MaybeOwnedRelay<'n>>The relays in the path.
netdir: &'n NetDirThe network directory.
vanguards: &'a VanguardMgr<RT>The vanguard manager.
rng: &'a mut RAn RNG for selecting vanguards and middle relays.
last_hop_kind: HopKindThe HopKind of the last hop in the path.
Implementations
Sourceimpl<'n, 'a, RT: Runtime, R: Rng> PathBuilder<'n, 'a, RT, R>
impl<'n, 'a, RT: Runtime, R: Rng> PathBuilder<'n, 'a, RT, R>
Sourcepub(super) fn new(
rng: &'a mut R,
netdir: &'n NetDir,
vanguards: &'a VanguardMgr<RT>,
l1_guard: MaybeOwnedRelay<'n>,
) -> Self
pub(super) fn new( rng: &'a mut R, netdir: &'n NetDir, vanguards: &'a VanguardMgr<RT>, l1_guard: MaybeOwnedRelay<'n>, ) -> Self
Create a new PathBuilder.
Sourcepub(super) fn add_vanguard(
self,
selector: &RelaySelector<'n>,
layer: Layer,
) -> Result<Self>
pub(super) fn add_vanguard( self, selector: &RelaySelector<'n>, layer: Layer, ) -> Result<Self>
Extend the path with a vanguard.
Sourcepub(super) fn add_middle(self, selector: &RelaySelector<'n>) -> Result<Self>
pub(super) fn add_middle(self, selector: &RelaySelector<'n>) -> Result<Self>
Extend the path with a middle relay.
Sourcepub(super) fn build(self) -> Result<TorPath<'n>>
pub(super) fn build(self) -> Result<TorPath<'n>>
Return a TorPath built using the hops from this PathBuilder.
Sourcefn add_hop(
&mut self,
hop: MaybeOwnedRelay<'n>,
hop_kind: HopKind,
) -> StdResult<(), Bug>
fn add_hop( &mut self, hop: MaybeOwnedRelay<'n>, hop_kind: HopKind, ) -> StdResult<(), Bug>
Try to append hop to the end of the path.
This also causes the PathBuilder to transition to the state represented by hop_kind,
if the transition is valid.
Returns an error if the hop_kind is incompatible with the HopKind of the last hop.
Sourcefn update_last_hop_kind(&mut self, kind: HopKind) -> StdResult<(), Bug>
fn update_last_hop_kind(&mut self, kind: HopKind) -> StdResult<(), Bug>
Transition to the state specified by kind.
The state of the PathBuilder is represented by the HopKind of its last hop.
This function should be called whenever a new hop is added
(e.g. in add_hop), to set the current state to the
HopKind of the new hop.
Not all transitions are valid. The permissible state transitions are:
G -> L2L2 -> L3L2 -> ML3 -> M