Struct tor_circmgr::ClockSkewEvents
source · pub struct ClockSkewEvents { /* private fields */ }Expand description
A stream of SkewEstimate events.
Note that this stream can be lossy: if multiple events trigger before you read from it, you will only get the most recent estimate.
Implementations
sourceimpl ClockSkewEvents
impl ClockSkewEvents
sourcepub fn get(&self) -> Option<SkewEstimate>
pub fn get(&self) -> Option<SkewEstimate>
Return our best estimate of our current clock skew, based on reports from the guards and fallbacks we have contacted.