pub struct PathConfig {
ipv4_subnet_family_prefix: u8,
ipv6_subnet_family_prefix: u8,
pub(crate) long_lived_ports: HashSet<u16>,
pub(crate) reachable_addrs: Vec<AddrPortPattern>,
}Expand description
Rules for building paths over the network.
This type is immutable once constructed. To build one, use
PathConfigBuilder, or deserialize it from a string.
You may change the PathConfig on a running Arti client. Doing so changes paths that are constructed in the future, and prevents requests from being attached to existing circuits, if the configuration has become more restrictive.
Fields
ipv4_subnet_family_prefix: u8Set the length of a bit-prefix for a default IPv4 subnet-family.
Any two relays will be considered to belong to the same family if their IPv4 addresses share at least this many initial bits.
ipv6_subnet_family_prefix: u8Set the length of a bit-prefix for a default IPv6 subnet-family.
Any two relays will be considered to belong to the same family if their IPv6 addresses share at least this many initial bits.
long_lived_ports: HashSet<u16>A set of ports that need to be sent over Stable circuits.
reachable_addrs: Vec<AddrPortPattern>The set of addresses to which we’re willing to make direct connections.
Implementations
Sourceimpl PathConfig
impl PathConfig
Sourcepub fn builder() -> PathConfigBuilder
pub fn builder() -> PathConfigBuilder
Return a new [PathConfig Builder] to construct an instance of this type.
Sourceimpl PathConfig
impl PathConfig
Sourcepub fn subnet_config(&self) -> SubnetConfig
pub fn subnet_config(&self) -> SubnetConfig
Return a subnet configuration based on these rules.
Sourcepub(crate) fn at_least_as_permissive_as(&self, other: &Self) -> bool
pub(crate) fn at_least_as_permissive_as(&self, other: &Self) -> bool
Return true if this configuration is at least as permissive as other.
In other words, in other words, return true if every circuit permitted
by other would also be permitted by this configuration.
We use this function to decide when circuits must be discarded. Therefore, it is okay to return “false” inaccurately, but we should never return “true” inaccurately.
Sourcepub(crate) fn build_guard_filter(&self) -> GuardFilter
pub(crate) fn build_guard_filter(&self) -> GuardFilter
Return a new GuardFilter reflecting the rules in this configuration.
Sourcepub(crate) fn relay_selection_config(&self) -> RelaySelectionConfig<'_>
pub(crate) fn relay_selection_config(&self) -> RelaySelectionConfig<'_>
Return a new RelaySelectionConfig reflecting the rules in this
configuration.