pub(crate) struct Estimator {
inner: Mutex<Box<dyn TimeoutEstimator + Send + 'static>>,
}Expand description
A timeout estimator that can change its inner implementation and share its implementation among multiple threads.
Fields
inner: Mutex<Box<dyn TimeoutEstimator + Send + 'static>>The estimator we’re currently using.
Implementations
Sourceimpl Estimator
impl Estimator
Sourcepub(crate) fn from_storage(
storage: &DynStorageHandle<ParetoTimeoutState>,
) -> Self
pub(crate) fn from_storage( storage: &DynStorageHandle<ParetoTimeoutState>, ) -> Self
Create this estimator based on the values stored in storage, and whether
this storage is read-only.
Sourcepub(crate) fn upgrade_to_owning_storage(
&self,
storage: &DynStorageHandle<ParetoTimeoutState>,
)
pub(crate) fn upgrade_to_owning_storage( &self, storage: &DynStorageHandle<ParetoTimeoutState>, )
Assuming that we can read and write to storage, replace our state with
a new state that estimates timeouts.
Sourcepub(crate) fn reload_readonly_from_storage(
&self,
storage: &DynStorageHandle<ParetoTimeoutState>,
)
pub(crate) fn reload_readonly_from_storage( &self, storage: &DynStorageHandle<ParetoTimeoutState>, )
Replace the contents of this estimator with a read-only state estimator
based on the contents of storage.
Sourcepub(crate) fn note_hop_completed(&self, hop: u8, delay: Duration, is_last: bool)
pub(crate) fn note_hop_completed(&self, hop: u8, delay: Duration, is_last: bool)
Record that a given circuit hop has completed.
The hop number is a zero-indexed value for which hop just completed.
The delay value is the amount of time after we first launched the
circuit.
If this is the last hop of the circuit, then is_last is true.
Sourcepub(crate) fn note_circ_timeout(&self, hop: u8, delay: Duration)
pub(crate) fn note_circ_timeout(&self, hop: u8, delay: Duration)
Record that a circuit failed to complete because it took too long.
The hop number is a the number of hops that were successfully
completed.
The delay number is the amount of time after we first launched the
circuit.
Sourcepub(crate) fn timeouts(&self, action: &Action) -> (Duration, Duration)
pub(crate) fn timeouts(&self, action: &Action) -> (Duration, Duration)
Return the current estimation for how long we should wait for a given
Action to complete.
This function should return a 2-tuple of (timeout, abandon)
durations. After timeout has elapsed since circuit launch,
the circuit should no longer be used, but we should still keep
building it in order see how long it takes. After abandon
has elapsed since circuit launch, the circuit should be
abandoned completely.
Sourcepub(crate) fn learning_timeouts(&self) -> bool
pub(crate) fn learning_timeouts(&self) -> bool
Return true if we’re currently trying to learn more timeouts by launching testing circuits.
Sourcepub(crate) fn update_params(&self, params: &NetParameters)
pub(crate) fn update_params(&self, params: &NetParameters)
Replace the network parameters used by this estimator (if any)
with ones derived from params.
Sourcepub(crate) fn save_state(
&self,
storage: &DynStorageHandle<ParetoTimeoutState>,
) -> Result<()>
pub(crate) fn save_state( &self, storage: &DynStorageHandle<ParetoTimeoutState>, ) -> Result<()>
Store any state associated with this timeout estimator into storage.