BoundedDeque

Struct BoundedDeque 

struct BoundedDeque<T> {
    inner: VecDeque<T>,
    limit: usize,
}
A wrapper around VecDeque<T> that prevents more a certain number of entries from being inserted.

Fields

inner: VecDeque<T>

The underlying VecDeque.

We could use a SmallVec or an array instead, but that would require reimplementing more of VecDeque.

limit: usize

The maximum number of elements to permit.

Implementations

Source

impl<T> BoundedDeque<T>

fn new(limit: usize) -> Self

Construct a new empty BoundedDeque, limited to limit entries.

fn clear(&mut self)

Remove every entry from this BoundedDeque.

fn len(&self) -> usize

Return the number of entries in this BoundedDeque.

fn push_back(&mut self, item: T) -> Option<T>

Add a new entry to the back of this BoundedDeque.

If the deque was at its limit, pop and return the entry at the front.

fn iter(&self) -> impl Iterator<Item = &T>

Return an iterator over the entries in this BoundedDeque, from front to back.

fn set_max_len(&mut self, new_limit: usize)

Replace the maximum number of observations in this BoundedDeque.

Unlike the equivalent method in the old BoundedVecDeque crate, if the new limit is smaller than the previous limit, this method will remove the oldest items from the queue

  • that is, the ones from the front.