pub(crate) struct Params {
use_estimates: bool,
min_observations: u16,
significant_hop: u8,
timeout_quantile: f64,
abandon_quantile: f64,
default_thresholds: (Duration, Duration),
n_modes_for_xm: usize,
success_history_len: usize,
reset_after_timeouts: usize,
min_timeout: Duration,
}Expand description
A set of parameters determining the behavior of a ParetoTimeoutEstimator.
These are typically derived from a set of consensus parameters.
Fields
use_estimates: boolShould we use our estimates when deciding on circuit timeouts.
When this is false, our timeouts are fixed to the default.
min_observations: u16How many observations must we have made before we can use our Pareto estimators to guess a good set of timeouts?
significant_hop: u8Which hop is the “significant hop” we should use when recording circuit build times? (Watch out! This is zero-indexed.)
timeout_quantile: f64A quantile (in range [0.0,1.0]) describing a point in the Pareto distribution to use when determining when a circuit should be treated as having “timed out”.
(A “timed out” circuit continues building for measurement purposes, but can’t be used for traffic.)
abandon_quantile: f64A quantile (in range [0.0,1.0]) describing a point in the Pareto distribution to use when determining when a circuit should be “abandoned”.
(An “abandoned” circuit is stopped entirely, and not included in measurements.
default_thresholds: (Duration, Duration)Default values to return from the timeouts function when we
have no observations.
n_modes_for_xm: usizeNumber of histogram buckets to use when determining the Xm estimate.
(See History::estimate_xm for details.)
success_history_len: usizeHow many entries do we record in our success/timeout history?
reset_after_timeouts: usizeHow many timeouts should we allow in our success/timeout history before we assume that network has changed in a way that makes our estimates completely wrong?
min_timeout: DurationMinimum base timeout to ever infer or return.