pub(crate) struct ParetoTimeoutEstimator {
history: History,
timeouts: Option<(Duration, Duration)>,
fallback_timeouts: (Duration, Duration),
p: Params,
}Expand description
Tor’s default circuit build timeout estimator.
This object records a set of observed circuit build times, and uses it to determine good values for how long we should allow circuits to build.
For full details of the algorithms used, see
path-spec.txt.
Fields
history: HistoryOur observations for circuit build times and success/failure history.
timeouts: Option<(Duration, Duration)>Our most recent timeout estimate, if we have one that is up-to-date.
(We reset this to None whenever we get a new observation.)
fallback_timeouts: (Duration, Duration)The timeouts that we use when we do not have sufficient observations to conclude anything about our circuit build times.
These start out as p.default_thresholds, but can be adjusted
depending on how many timeouts we’ve been seeing.
p: ParamsA set of parameters to use in computing circuit build timeout estimates.
Implementations
Sourceimpl ParetoTimeoutEstimator
impl ParetoTimeoutEstimator
Sourcefn from_history(history: History) -> Self
fn from_history(history: History) -> Self
Construct a new ParetoTimeoutEstimator from the provided history object.
Sourcepub(crate) fn from_state(state: ParetoTimeoutState) -> Self
pub(crate) fn from_state(state: ParetoTimeoutState) -> Self
Create a new ParetoTimeoutEstimator based on a loaded ParetoTimeoutState.
Sourcefn base_timeouts(&mut self) -> (Duration, Duration)
fn base_timeouts(&mut self) -> (Duration, Duration)
Compute an unscaled basic pair of timeouts for a circuit of the “normal” length.
Return a cached value if we have no observations since the last time this function was called.