pub(crate) struct ParetoTimeoutState {
version: usize,
histogram: Vec<(MsecDuration, u16)>,
current_timeout: Option<MsecDuration>,
unknown_fields: HashMap<String, JsonValue>,
}Expand description
An object used to serialize our timeout history for persistent state.
Fields
version: usizeA version field used to help encoding and decoding.
histogram: Vec<(MsecDuration, u16)>A record of observed timeouts, as returned by sparse_histogram().
current_timeout: Option<MsecDuration>The current timeout estimate: kept for reference.
unknown_fields: HashMap<String, JsonValue>Fields from the state file that was used to make this ParetoTimeoutState that
this version of Arti doesn’t understand.
Implementations
Sourceimpl ParetoTimeoutState
impl ParetoTimeoutState
Sourcepub(crate) fn latest_estimate(&self) -> Option<Duration>
pub(crate) fn latest_estimate(&self) -> Option<Duration>
Return the latest base timeout estimate, as recorded in this state.