pub(crate) struct ReadonlyTimeoutEstimator {
using_estimates: bool,
latest_timeout: Option<Duration>,
default_timeout: Duration,
}Expand description
A timeout estimator based on reading timeouts that another timeout estimator is computing, in another process.
Fields
using_estimates: boolAre we using the timeouts?
latest_timeout: Option<Duration>Latest estimate from the persistent state.
default_timeout: DurationTimeout to use if we don’t have a computed timeout.
Implementations
Sourceimpl ReadonlyTimeoutEstimator
impl ReadonlyTimeoutEstimator
Sourcepub(crate) fn from_state(s: &ParetoTimeoutState) -> Self
pub(crate) fn from_state(s: &ParetoTimeoutState) -> Self
Create a new ReadonlyTimeoutEstimator, based on persistent state
Sourcepub(crate) fn update_from_state(&mut self, s: &ParetoTimeoutState)
pub(crate) fn update_from_state(&mut self, s: &ParetoTimeoutState)
Update this estimator based on a newly read state.