pub struct ClientDirTunnel {
tunnel: Arc<ClientTunnel>,
}Expand description
A client directory tunnel. This is always single path.
Fields
tunnel: Arc<ClientTunnel>The protocol level tunnel.
Implementations
Sourceimpl ClientDirTunnel
impl ClientDirTunnel
Sourcefn tunnel_ref(&self) -> &Arc<ClientTunnel>
fn tunnel_ref(&self) -> &Arc<ClientTunnel>
Return a reference to the underlying tunnel.
Note: The “tunnel” name is hardcoded here. If it becomes an annoyance, we could make it as a meta value of the deftly declaration.
Sourcepub fn last_hop(&self) -> Result<TargetHop>
pub fn last_hop(&self) -> Result<TargetHop>
Return a TargetHop representing precisely the last hop of the circuit as in set as a
HopLocation with its id and hop number.
Return an error if there is no last hop.
Sourcepub fn terminate(&self)
pub fn terminate(&self)
Shutdown the tunnel meaning this sends a shutdown command to the underlying circuit reactor which will stop asynchronously.
Note that it is not necessary to use this method as in if the tunnel reference is dropped, the circuit will close automatically.
Sourcepub async fn send_raw_msg(&self, msg: AnyRelayMsg, hop: TargetHop) -> Result<()>
pub async fn send_raw_msg(&self, msg: AnyRelayMsg, hop: TargetHop) -> Result<()>
Send raw message.
Sourcepub async fn start_conversation(
&self,
msg: Option<AnyRelayMsg>,
reply_handler: impl MsgHandler + Send + 'static,
hop: TargetHop,
) -> Result<Conversation<'_>>
pub async fn start_conversation( &self, msg: Option<AnyRelayMsg>, reply_handler: impl MsgHandler + Send + 'static, hop: TargetHop, ) -> Result<Conversation<'_>>
Start an ad-hoc protocol exchange to the specified hop on this tunnel.
See ClientTunnel::start_conversation
documentation for more details.
Sourceimpl ClientDirTunnel
impl ClientDirTunnel
Sourcepub async fn begin_dir_stream(&self) -> Result<DataStream>
pub async fn begin_dir_stream(&self) -> Result<DataStream>
Start a stream to the given address and port, using a BEGIN_DIR cell.
Sourceimpl ClientDirTunnel
impl ClientDirTunnel
Sourcefn circuit(&self) -> Result<&ClientCirc>
fn circuit(&self) -> Result<&ClientCirc>
Return a reference to the circuit of this tunnel.
Sourcepub async fn extend<T: CircTarget>(
&self,
target: &T,
params: CircParameters,
) -> Result<()>
pub async fn extend<T: CircTarget>( &self, target: &T, params: CircParameters, ) -> Result<()>
Extend the circuit to a new target last hop using the ntor v3 handshake.
TODO: Might want to pass which handshake type as a parameter so this function can be a catch all on all possible handshakes. For now, use ntor v3 for all the things.
Sourceimpl ClientDirTunnel
impl ClientDirTunnel
Sourcepub fn first_hop(&self) -> OwnedChanTarget
pub fn first_hop(&self) -> OwnedChanTarget
Return a description of the first hop of this circuit.
Sourcepub async fn first_hop_clock_skew(&self) -> Result<ClockSkew>
pub async fn first_hop_clock_skew(&self) -> Result<ClockSkew>
Get the clock skew claimed by the first hop of the circuit.