SupportedTunnelUsage

Enum SupportedTunnelUsage 

pub(crate) enum SupportedTunnelUsage {
    Dir,
    Exit {
        policy: ExitPolicy,
        isolation: Option<StreamIsolation>,
        country_code: Option<CountryCode>,
        all_relays_stable: bool,
    },
    NoUsage,
    HsOnly,
    DirSpecificTarget(OwnedChanTarget),
}
The purposes for which a circuit is usable.

This type should stay internal to the circmgr crate for now: we’ll probably want to refactor it a lot.

Variants



Dir

Usable for BEGINDIR-based non-anonymous directory connections



Exit

Usable to exit to a set of ports.

Fields

policy: ExitPolicy

Exit policy of the circuit

isolation: Option<StreamIsolation>

Isolation group the circuit is part of. None when the circuit is not yet assigned to an isolation group.

country_code: Option<CountryCode>

Country code the exit is in, or None if no country could be determined.

all_relays_stable: bool

Whether every relay in this circuit has the “Stable” flag.



NoUsage

This circuit is not suitable for any usage.



HsOnly

This circuit is for some hs-related usage. (It should never be given to the circuit manager; the HsPool code will handle it instead.)



DirSpecificTarget(OwnedChanTarget)

Use only for BEGINDIR-based non-anonymous directory connections to a particular target (which may not be in the netdir).

Implementations

Source

impl SupportedTunnelUsage

pub(crate) fn supports(&self, target: &TargetTunnelUsage) -> bool

Return true if this spec permits the usage described by other.

If this function returns true, then it is okay to use a circuit with this spec for the target usage described by other.

pub(crate) fn restrict_mut( &mut self, usage: &TargetTunnelUsage, ) -> Result<(), RestrictionFailed>

Change the value of this spec based on the circuit having been used for usage.

Returns an error and makes no changes to self if usage was not supported by this spec.

If this function returns Ok, the resulting spec will be contained by the original spec, and will support usage.

pub(crate) fn find_supported<'a, 'b, C: AbstractTunnel>( list: impl Iterator<Item = &'b mut OpenEntry<C>>, usage: &TargetTunnelUsage, ) -> Vec<&'b mut OpenEntry<C>>

Find all open circuits in list whose specifications permit usage.

pub(crate) fn channel_usage(&self) -> ChannelUsage

How the circuit will be used, for use by the channel

pub(crate) fn is_long_lived(&self) -> bool

True if this usage is compatible with a long-lived circuit.

(We leave long-lived circuits alive until they have been disused for a long time, and never expire them for being dirty.)