pub(crate) enum TargetTunnelUsage {
Dir,
Exit {
ports: Vec<TargetPort>,
isolation: StreamIsolation,
country_code: Option<CountryCode>,
require_stability: bool,
},
TimeoutTesting,
Preemptive {
port: Option<TargetPort>,
circs: usize,
require_stability: bool,
},
DirSpecificTarget(OwnedChanTarget),
HsCircBase {
compatible_with_target: Option<OwnedChanTarget>,
stem_kind: HsCircStemKind,
circ_kind: Option<HsCircKind>,
},
}Expand description
The purpose for which a circuit is being created.
This type should stay internal to the circmgr crate for now: we’ll probably want to refactor it a lot.
Variants
Dir
Use for BEGINDIR-based non-anonymous directory connections
Exit
Use to exit to one or more ports.
Fields
ports: Vec<TargetPort>List of ports the circuit has to allow.
If this list of ports is empty, then the circuit doesn’t need to support any particular port, but it still needs to be an exit.
isolation: StreamIsolationIsolation group the circuit shall be part of
country_code: Option<CountryCode>Restrict the circuit to only exits in the provided country code.
TimeoutTesting
For a circuit is only used for the purpose of building it.
Preemptive
For internal usage only: build a circuit preemptively, to reduce wait times.
Warning
This MUST NOT be used by code outside of the preemptive circuit predictor. In
particular, this usage doesn’t support stream isolation, so using it to ask for
circuits (for example, by passing it to get_or_launch) could be unsafe!
Fields
port: Option<TargetPort>A port the circuit has to allow, if specified.
If this is None, we just want a circuit capable of doing DNS resolution.
DirSpecificTarget(OwnedChanTarget)
Use for BEGINDIR-based non-anonymous directory connections to a particular target, and therefore to a specific relay (which need not be in any netdir).
HsCircBase
Used to build a circuit (currently always 3 hops) to serve as the basis of some onion-serivice-related operation.
Fields
compatible_with_target: Option<OwnedChanTarget>A target to avoid when constructing this circuit.
This target is not appended to the end of the circuit; rather, the circuit is built so that its relays are all allowed to share a circuit with this target (without, for example, violating any family restrictions).
stem_kind: HsCircStemKindThe kind of circuit stem to build.
circ_kind: Option<HsCircKind>If present, add additional rules to the stem so it can definitely be used as a circuit of this kind.
Implementations
Sourceimpl TargetTunnelUsage
impl TargetTunnelUsage
Sourcepub(crate) fn build_path<'a, R: Rng, RT: Runtime>(
&self,
rng: &mut R,
netdir: DirInfo<'a>,
guards: &GuardMgr<RT>,
vanguards: &VanguardMgr<RT>,
config: &PathConfig,
now: SystemTime,
) -> Result<(TorPath<'a>, SupportedTunnelUsage, Option<GuardMonitor>, Option<GuardUsable>)>
pub(crate) fn build_path<'a, R: Rng, RT: Runtime>( &self, rng: &mut R, netdir: DirInfo<'a>, guards: &GuardMgr<RT>, vanguards: &VanguardMgr<RT>, config: &PathConfig, now: SystemTime, ) -> Result<(TorPath<'a>, SupportedTunnelUsage, Option<GuardMonitor>, Option<GuardUsable>)>
Construct path for a given circuit purpose; return it and the usage that it actually supports.