fn owned_targets_equivalent(a: &OwnedChanTarget, b: &OwnedChanTarget) -> boolExpand description
Return true if a and b count as the same target for the purpose of
comparing DirSpecificTarget values.
fn owned_targets_equivalent(a: &OwnedChanTarget, b: &OwnedChanTarget) -> boolReturn true if a and b count as the same target for the purpose of
comparing DirSpecificTarget values.