pub struct TunnelBuilder<R: Runtime> {
builder: Arc<Builder<R, ClientTunnel>>,
path_config: MutCfg<PathConfig>,
storage: DynStorageHandle<ParetoTimeoutState>,
guardmgr: GuardMgr<R>,
vanguardmgr: Arc<VanguardMgr<R>>,
}Expand description
A factory object to build circuits.
A TunnelBuilder holds references to all the objects that are needed
to build circuits correctly.
In general, you should not need to construct or use this object yourself, unless you are choosing your own paths.
Fields
builder: Arc<Builder<R, ClientTunnel>>The underlying Builder object
path_config: MutCfg<PathConfig>Configuration for how to choose paths for circuits.
storage: DynStorageHandle<ParetoTimeoutState>State-manager object to use in storing current state.
guardmgr: GuardMgr<R>Guard manager to tell us which guards nodes to use for the circuits we build.
vanguardmgr: Arc<VanguardMgr<R>>The vanguard manager object used for HS circuits.
Implementations
Sourceimpl<R: Runtime> TunnelBuilder<R>
impl<R: Runtime> TunnelBuilder<R>
Sourcepub(crate) fn new(
runtime: R,
chanmgr: Arc<ChanMgr<R>>,
path_config: PathConfig,
storage: DynStorageHandle<ParetoTimeoutState>,
guardmgr: GuardMgr<R>,
vanguardmgr: VanguardMgr<R>,
) -> Self
pub(crate) fn new( runtime: R, chanmgr: Arc<ChanMgr<R>>, path_config: PathConfig, storage: DynStorageHandle<ParetoTimeoutState>, guardmgr: GuardMgr<R>, vanguardmgr: VanguardMgr<R>, ) -> Self
Construct a new TunnelBuilder.
Sourcepub(crate) fn path_config(&self) -> Arc<PathConfig>
pub(crate) fn path_config(&self) -> Arc<PathConfig>
Return this builder’s PathConfig.
Sourcepub(crate) fn set_path_config(&self, new_config: PathConfig)
pub(crate) fn set_path_config(&self, new_config: PathConfig)
Replace this builder’s PathConfig.
Sourcepub(crate) fn save_state(&self) -> Result<bool>
pub(crate) fn save_state(&self) -> Result<bool>
Flush state to the state manager if we own the lock.
Return Ok(true) if we saved, and Ok(false) if we didn’t hold the lock.
Sourcepub(crate) fn upgrade_to_owned_state(&self) -> Result<()>
pub(crate) fn upgrade_to_owned_state(&self) -> Result<()>
Replace our state with a new owning state, assuming we have storage permission.
Sourcepub(crate) fn reload_state(&self) -> Result<()>
pub(crate) fn reload_state(&self) -> Result<()>
Reload persistent state from disk, if we don’t have storage permission.
Sourcepub fn update_network_parameters(&self, p: &NetParameters)
pub fn update_network_parameters(&self, p: &NetParameters)
Reconfigure this builder using the latest set of network parameters.
(NOTE: for now, this only affects circuit timeout estimation.)
Sourcepub(crate) async fn build_owned(
&self,
path: OwnedPath,
params: &CircParameters,
guard_status: Arc<GuardStatusHandle>,
usage: ChannelUsage,
) -> Result<ClientTunnel>
pub(crate) async fn build_owned( &self, path: OwnedPath, params: &CircParameters, guard_status: Arc<GuardStatusHandle>, usage: ChannelUsage, ) -> Result<ClientTunnel>
Like build, but construct a new circuit from an OwnedPath.
Sourcepub async fn build(
&self,
path: &TorPath<'_>,
params: &CircParameters,
usage: ChannelUsage,
) -> Result<ClientTunnel>
pub async fn build( &self, path: &TorPath<'_>, params: &CircParameters, usage: ChannelUsage, ) -> Result<ClientTunnel>
Try to construct a new circuit from a given path, using appropriate timeouts.
This circuit is not automatically registered with any circuit manager; if you don’t hang on it it, it will automatically go away when the last reference is dropped.
Sourcepub(crate) fn learning_timeouts(&self) -> bool
pub(crate) fn learning_timeouts(&self) -> bool
Return true if this builder is currently learning timeout info.
Sourcepub(crate) fn vanguardmgr(&self) -> &Arc<VanguardMgr<R>>
pub(crate) fn vanguardmgr(&self) -> &Arc<VanguardMgr<R>>
Return a reference to this builder’s VanguardMgr.