pub struct CircuitTimingBuilder {
max_dirtiness: Option<Duration>,
disused_circuit_timeout: Option<Duration>,
request_timeout: Option<Duration>,
request_max_retries: Option<u32>,
request_loyalty: Option<Duration>,
hs_desc_fetch_attempts: Option<u32>,
hs_intro_rend_attempts: Option<u32>,
}Expand description
A builder to create an instance of CircuitTiming.
Fields
max_dirtiness: Option<Duration>In-progress value for max_dirtiness.
disused_circuit_timeout: Option<Duration>In-progress value for disused_circuit_timeout.
request_timeout: Option<Duration>In-progress value for request_timeout.
request_max_retries: Option<u32>In-progress value for request_max_retries.
request_loyalty: Option<Duration>In-progress value for request_loyalty.
hs_desc_fetch_attempts: Option<u32>In-progress value for hs_desc_fetch_attempts.
hs_intro_rend_attempts: Option<u32>In-progress value for hs_intro_rend_attempts.
Implementations
Sourceimpl CircuitTimingBuilder
impl CircuitTimingBuilder
Sourcepub fn max_dirtiness(&mut self, val: Duration) -> &mut Self
pub fn max_dirtiness(&mut self, val: Duration) -> &mut Self
Provide a value for max_dirtiness.
If no value is provided for max_dirtiness, build will use Default::default().
max_dirtiness
How long after a circuit has first been used should we give it out for new requests?
This setting applies to circuits without strong isolation.
See also disused_circuit_timeout
Sourcepub fn disused_circuit_timeout(&mut self, val: Duration) -> &mut Self
pub fn disused_circuit_timeout(&mut self, val: Duration) -> &mut Self
Provide a value for disused_circuit_timeout.
If no value is provided for disused_circuit_timeout, build will use Default::default().
disused_circuit_timeout
How long after a circuit has become disused should we discard it?
This setting applies to circuits with strong isolation.
See also max_dirtiness
Sourcepub fn request_timeout(&mut self, val: Duration) -> &mut Self
pub fn request_timeout(&mut self, val: Duration) -> &mut Self
Sourcepub fn request_max_retries(&mut self, val: u32) -> &mut Self
pub fn request_max_retries(&mut self, val: u32) -> &mut Self
Sourcepub fn request_loyalty(&mut self, val: Duration) -> &mut Self
pub fn request_loyalty(&mut self, val: Duration) -> &mut Self
Sourcepub fn hs_desc_fetch_attempts(&mut self, val: u32) -> &mut Self
pub fn hs_desc_fetch_attempts(&mut self, val: u32) -> &mut Self
Sourcepub fn hs_intro_rend_attempts(&mut self, val: u32) -> &mut Self
pub fn hs_intro_rend_attempts(&mut self, val: u32) -> &mut Self
Sourceimpl CircuitTimingBuilder
impl CircuitTimingBuilder
Sourcepub fn build(&self) -> Result<CircuitTiming, ConfigBuildError>
pub fn build(&self) -> Result<CircuitTiming, ConfigBuildError>
Try to construct a new CircuitTiming from the fields set in this builder.
Return an error if any required field is missing, or is set to something invalid.