pub(crate) fn hs_stem_terminal_hop_usage(kind: Option<HsCircKind>) -> RelayUsageExpand description
Return the usage that we should use when selecting the last hop of a stem circuit.
If kind is provided, we need to make sure that the last hop will yield a stem circuit
that’s fit for that kind of circuit.